@@ -14,14 +14,13 @@
 """
 
 
-import sys
 import unittest
 
 from tests.utils.base_unittest import BaseCreoleTest
 
 from creole import html2creole
-from creole.html2creole import RAISE_UNKNOWN_NODES, HTML_MACRO_UNKNOWN_NODES, \
-                                                            ESCAPE_UNKNOWN_NODES
+from creole.shared.unknown_tags import raise_unknown_node, use_html_macro, \
+                            escape_unknown_nodes, transparent_unknown_nodes
 
 
 class TestHtml2Creole(unittest.TestCase):
@@ -35,8 +34,8 @@ class TestHtml2Creole(unittest.TestCase):
 
 class TestHtml2CreoleMarkup(BaseCreoleTest):
 
-    def assertCreole(self, raw_markup, raw_html, debug=False, **kwargs):
-        self.assert_html2Creole(raw_markup, raw_html, debug=debug, **kwargs)
+#    def assertCreole(self, raw_markup, raw_html, debug=False, **kwargs):
+#        self.assert_html2creole(raw_markup, raw_html, debug=debug, **kwargs)
 
     #--------------------------------------------------------------------------
 
@@ -44,7 +43,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
         """
         Some other html tags -> convert.
         """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             **Bold text**
             **Big text**
             //em tag//
@@ -56,23 +55,44 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
             <i>italic</i></p>
         """)
 
+    #--------------------------------------------------------------------------
+
     def test_raise_unknown_node(self):
         """
-        Test creole.html2creole.RAISE_UNKNOWN_NODES mode:
+        Test creole.html2creole.raise_unknown_node callable:
         Raise NotImplementedError on unknown tags.
         """
         self.assertRaises(NotImplementedError,
             html2creole,
             html_string=u"<unknwon>",
-            unknown_emit=RAISE_UNKNOWN_NODES
+            emitter_kwargs={"unknown_emit":raise_unknown_node}
+        )
+
+    def test_use_html_macro(self):
+        """
+        Test creole.html2creole.use_html_macro callable:
+        Use the <<html>> macro to mask unknown tags.
+        """
+        self.assert_html2creole(r"""
+            111 <<html>><unknown><</html>>foo<<html>></unknown><</html>> 222
+            333<<html>><unknown foo1="bar1" foo2="bar2"><</html>>foobar<<html>></unknown><</html>>444
+
+            555<<html>><unknown /><</html>>666
+        """, """
+            <p>111 <unknown>foo</unknown> 222<br />
+            333<unknown foo1="bar1" foo2="bar2">foobar</unknown>444</p>
+
+            <p>555<unknown />666</p>
+        """,
+            emitter_kwargs={"unknown_emit":use_html_macro}
         )
 
     def test_escape_unknown_nodes(self):
         """
-        Test creole.html2creole.ESCAPE_UNKNOWN_NODES mode:
+        Test creole.html2creole.escape_unknown_nodes callable:
         All unknown tags should be escaped.
         """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             111 &lt;unknown&gt;foo&lt;/unknown&gt; 222
             333&lt;unknown foo1="bar1" foo2="bar2"&gt;foobar&lt;/unknown&gt;444
 
@@ -83,9 +103,66 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 
             <p>555<unknown />666</p>
         """,
-            unknown_emit=ESCAPE_UNKNOWN_NODES
+            emitter_kwargs={"unknown_emit":escape_unknown_nodes}
         )
 
+    def test_escape_unknown_nodes2(self):
+        """ TODO: Seems that <script> would not be handled right. """
+        self.assert_html2creole(r"""
+            &lt;script&gt;var js_sha_link='&lt;p&gt;***&lt;/p&gt;&lt;/script&gt;';
+        """, """
+            <script>
+            var js_sha_link='<p>***</p>';
+            </script>
+        """,
+            emitter_kwargs={"unknown_emit":escape_unknown_nodes}
+        )
+
+    def test_transparent_unknown_nodes(self):
+        """
+        Test creole.html2creole.transparent_unknown_nodes callable:
+        All unknown tags should be "transparent" and show only
+        their child nodes' content.
+        """
+        self.assert_html2creole(r"""
+            //baz//, **quux**
+        """, """
+            <form class="foo" id="bar"><label><em>baz</em></label>, <strong>quux</strong></form>
+        """, emitter_kwargs={"unknown_emit":transparent_unknown_nodes}
+        )
+
+    def test_transparent_unknown_nodes2(self):
+        """ TODO: Seems that <script> would not be handled right. """
+        self.assert_html2creole(r"""
+            FOO
+            BAR
+        """, """
+            <p>FOO</p>
+            <script>
+            var js_sha_link='<em>STRONG</em>';
+            </script>
+            <p>BAR</p>
+        """, emitter_kwargs={"unknown_emit":transparent_unknown_nodes}
+        )
+
+    def test_transparent_unknown_nodes_block_elements(self):
+        """
+        Test that block elements insert linefeeds into the stream.
+        """
+        self.assert_html2creole(r"""
+            //baz//,
+
+            **quux**
+
+            spam, ham, and eggs
+        """, """
+            <div><em>baz</em>,</div> <fieldset><strong>quux</strong></fieldset>
+            <span>spam, </span><label>ham, </label>and eggs
+        """, emitter_kwargs={"unknown_emit":transparent_unknown_nodes}
+        )
+
+    #--------------------------------------------------------------------------        
+
     def test_entities(self):
         """
         Test html entities.
@@ -95,7 +172,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
         Box Drawing:
             http://pylucid.org/_command/144/DecodeUnicode/display/66/
         """
-        self.assertCreole(u"""
+        self.assert_html2creole(u"""
             * less-than sign: < < <
             * greater-than sign: > > >
             * copyright sign: © ©
@@ -113,14 +190,14 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 
     def test_html_entity_nbsp(self):
         """ Non breaking spaces is not in htmlentitydefs """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             a non braking space: [ ] !
         """, """
             <p>a non braking space: [&nbsp;] !</p>
         """)
 
     def test_html_entity_in_pre(self):
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             {{{<code>{% lucidTag RSS url="http url" %}</code>}}}
         """, """
             <pre><code>&#x7B;% lucidTag RSS url="http url" %&#x7D;</code></pre>
@@ -131,19 +208,19 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
         Test a unknown html entity.
         FIXME: What sould happend?
         """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             copy&paste
         """, """
             <p>copy&paste</p>
         """)
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             [[/url/|Search & Destroy]]
         """, """
             <a href="/url/">Search & Destroy</a>
         """)
 
     def test_tbody_table(self):
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             Ignore 'tbody' tag in tables:
             
             |= Headline 1 |= Headline 2 |
@@ -168,7 +245,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 
     def test_p_table(self):
         """ strip <p> tags in table cells """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             | cell one | cell two\\new line |
         """, """
             <table>
@@ -181,7 +258,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 
     def test_image(self):
         """ test image tag with different alt/title attribute """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             {{foobar1.jpg|foobar1.jpg}}
             {{/foobar2.jpg|foobar2.jpg}}
             {{/path1/path2/foobar3.jpg|foobar3.jpg}}
@@ -194,11 +271,14 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
             <img src="/path1/path2/foobar3.jpg" /><br />
             <img src="/foobar4.jpg" alt="It's foobar 4" /><br />
             <img src="/foobar5.jpg" title="It's foobar 5" /><br />
-            <img src="/foobar6.jpg" alt="short name" title="a long picture title" /></p>
+            <img src="/foobar6.jpg" alt="short name" title="a long picture title" /><br />
+            <img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUA
+            AAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO
+            9TXL0Y4OHwAAAABJRU5ErkJggg==" alt="data uri should be disallowed" /></p>
         """)
 
     def test_non_closed_br(self):
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             one
             two
         """, """
@@ -207,7 +287,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
         """)
 
     def test_explicit_closed_br(self):
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             one
             two
         """, """
@@ -219,7 +299,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
         """
         http://code.google.com/p/python-creole/issues/detail?id=16
         """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             **foo**
             
             * one
@@ -228,19 +308,49 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
         """)
 
     def test_empty_tags_are_not_escaped(self):
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             //baz//, **quux**
         """, """
             <div class="foo" id="bar"><span><em>baz</em></span>, <strong>quux</strong></div>
         """)
 
+    def test_nested_listsitems_with_paragraph(self):
+        self.assert_html2creole(u"""
+            * item 1
+            ** subitem 1.1
+            *** subsubitem 1.1.1
+            *** subsubitem 1.1.2
+            ** subitem 1.2
+            * item 2
+            ** subitem 2.1
+            """, """
+            <ul>
+                <li><p>item 1</p>
+                    <ul>
+                        <li><p>subitem 1.1</p>
+                            <ul>
+                                <li>subsubitem 1.1.1</li>
+                                <li>subsubitem 1.1.2</li>
+                            </ul>
+                        </li>
+                            <li><p>subitem 1.2</p></li>
+                    </ul>
+                </li>
+                <li><p>item 2</p>
+                    <ul>
+                        <li>subitem 2.1</li>
+                    </ul>
+                </li>
+            </ul>
+        """)
+
     #--------------------------------------------------------------------------
     # TODOs:
 
 
     def test_newline_before_headline(self):
         """ TODO: http://code.google.com/p/python-creole/issues/detail?id=16#c5 """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             **foo**
             
             = one
@@ -249,24 +359,20 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
             <h1>one</h1>
         """)#, debug=True)
 
-    def test_cross_lines(self):
-        """ TODO: bold/italics cross lines
-        see: http://code.google.com/p/python-creole/issues/detail?id=13 
-        """
-        self.assertCreole(r"""
-            Bold and italics should //be
-            able// to **cross
-            lines.**
-        """, """
-            <p>Bold and italics should <i>be<br />
-            able</i> to <strong>cross<br />
-            lines.</strong></p>
-        """)
+    def test_newlines_after_headlines(self):
+        self.assert_html2creole(r"""
+            = Headline news
 
+            [[http://google.com|The googlezor]] is a big bad mother.
+        """, """
+            <h1>Headline news</h1>
+
+            <p><a href="http://google.com">The googlezor</a> is a big bad mother.</p>
+        """)
 
     def test_no_space_before_blocktag(self):
         """ TODO: Bug in html2creole.strip_html(): Don't add a space before/after block tags """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             **foo**
             
             * one
@@ -278,7 +384,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 
     def test_format_in_a_text(self):
         """ TODO: http://code.google.com/p/python-creole/issues/detail?id=4 """
-        self.assertCreole(r"""
+        self.assert_html2creole(r"""
             **[[/url/|title]]**
         """, """
             <a href="/url/"><strong>title</strong></a>
@@ -286,7 +392,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 
 
 #    def test_links(self):
-#        self.assertCreole(r"""
+#        self.assert_html2creole(r"""
 #            test link: '[[internal links|link A]]' 1 and
 #            test link: '[[http://domain.tld|link B]]' 2.
 #        """, """
@@ -295,7 +401,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 #        """)
 #
 #    def test_images(self):
-#        self.assertCreole(r"""
+#        self.assert_html2creole(r"""
 #            a {{/image.jpg|JPG pictures}} and
 #            a {{/image.jpeg|JPEG pictures}} and
 #            a {{/image.gif|GIF pictures}} and
@@ -312,7 +418,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 #        """)
 #
 #    def test_nowiki1(self):
-#        self.assertCreole(r"""
+#        self.assert_html2creole(r"""
 #            this:
 #            {{{
 #            //This// does **not** get [[formatted]]
@@ -347,7 +453,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 
 #
 #    def test_horizontal_rule(self):
-#        self.assertCreole(r"""
+#        self.assert_html2creole(r"""
 #            one
 #            ----
 #            two
@@ -361,7 +467,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 #        """
 #        FIXME: Two newlines between a list and the next paragraph :(
 #        """
-#        self.assertCreole(r"""
+#        self.assert_html2creole(r"""
 #            ==== List a:
 #            * a1 item
 #            ** a1.1 Force\\linebreak
@@ -422,7 +528,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 #
 #    def test_list2(self):
 #        """ Bold, Italics, Links, Pre in Lists """
-#        self.assertCreole(r"""
+#        self.assert_html2creole(r"""
 #            * **bold** item
 #            * //italic// item
 #
@@ -445,7 +551,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 #    # TODO:
 #
 #    def test_escape_char(self):
-#        self.assertCreole(r"""
+#        self.assert_html2creole(r"""
 #            ~#1
 #            http://domain.tld/~bar/
 #            ~http://domain.tld/
@@ -460,7 +566,7 @@ class TestHtml2CreoleMarkup(BaseCreoleTest):
 #        """)
 
 if __name__ == '__main__':
-    unittest.main()
-#if __name__ == '__main__':
-#    suite = unittest.TestLoader().loadTestsFromTestCase(TestHtml2Creole)
-#    unittest.TextTestRunner().run(suite)
+    unittest.main(
+#        defaultTest="TestHtml2CreoleMarkup.test_nested_listsitems_with_paragraph"
+    )
+
